/*****************************************************************************
 *                                                                           *
 *                Licensed Materials - Property of IBM Corp.                 *
 *                  5648-C60 (c) Copyright IBM Corp. 2000                    *
 *                                                                           *
 *       MQJMS Standalone Application Server Facilities (ASF) Samples        *
 *                                                                           *
 *                                                                           *
 *  FILENAME:     ASFClient3.java                                            *
 *                                                                           *
 *  DESCRIPTION:  The third ASF client application sample. It is a simple    *
 *                publish/subscribe version of the ASFClient1 sample.        *
 *                It sets up a single non-durable Topic-based Connection-    *
 *                Consumer.                                                  *
 *                                                                           *
 *                Further information about this class and its usage can be  *
 *                found in the 'Websphere MQ Using Java' documentation.      *
 *                                                                           *
 *  USAGE:        java ASFClient3 [options]                                  *
 *                                                                           *
 *                This application accepts the -nojndi, -qm, -t, -icf, -url, *
 *                -tcfLookup and -tfLookup options as described in the       *
 *                TopicLoad.java source and the documentation.               *
 *                                                                           *
 *                Additionally, it can take '-poolsize <poolSize>' and       *
 *                '-batchsize <batchSize>', rather like the previous two     *
 *                client samples.                                            *
 *                                                                           *
 *  HISTORY:      010131 - First released as part of SupportPac MA88 v2      *
 *                                                                           *
 *****************************************************************************/

import javax.jms.*;
import javax.naming.*;
import java.util.Hashtable;

public class ASFClient3 {
   // These are the defaults for the parameters which can be over-ridden
   // on the command line

   // Will we use JNDI for administered object creation?
   private static boolean useJNDI   = true;
   // JNDI parameters
   private static String  tcfLookup = "cn=tcf";
   private static String  tLookup   = "cn=t";
   private static String  icf       = "com.sun.jndi.ldap.LdapCtxFactory";
   private static String  url       = "ldap://localhost/o=ibm,c=us";
   // Non-JNDI parameters
   private static String  qMgrName  = "";
   private static String  tName     = "MQJMS/ASF/TopicLoad";
   // Other parameters
   private static int     poolSize  = 5;
   private static int     batchSize = 10;

   /**************************************************************************
    *  main - Application entry point                                        *
    **************************************************************************/
   public static void main( String[] args ) throws JMSException
   {
      // Parse the command line
      parseArguments(args);

      // Obtain the two administered objects we need
      Object[] adminObjects = new Object[2];
      obtainAdminObjects( adminObjects );
      TopicConnectionFactory tcf = (TopicConnectionFactory)adminObjects[0];
      Topic t = (Topic)adminObjects[1];

      // Obtain a TopicConnection
      TopicConnection conn = tcf.createTopicConnection();

      // Create a MessageListenerFactory; in this case, it is simply a
      // CountingMessageListenerFactory, to which we give the option name
      // "ReceiveAllMessages" for identification purposes
      CountingMessageListenerFactory mlf = new CountingMessageListenerFactory();
      mlf.setName("ReceiveAllMessages");

      // Create a ServerSessionPool using our simple MyServerSessionPool
      // implementation
      MyServerSessionPool ssp =
           new MyServerSessionPool( conn,         // The JMS Connection to use
                                    poolSize,           // The pool's capacity
                                    Session.AUTO_ACKNOWLEDGE,  // The ack mode
                                    mlf );    // MessageListenerFactory to use

      // Finally, get a ConnectionConsumer and then start things going by
      // starting the connection
      System.out.print("Creating ConnectionConsumer... ");
      conn.createConnectionConsumer( t,     // The Destination to consume from
                                     null,      // No message selector applied
                                     ssp,      // The ServerSessionPool to use
                                     batchSize );// Maximum message batch size
      System.out.print("Done.\nStarting Connection... ");
      conn.start();
      System.out.println("Done.");

      // Now, every ten seconds for a minute, get the message listener
      // factory to print out its stats
      for( int i=0; i<6; i++ ) {
         try {
            // Application thread sleeps for 10 seconds
            Thread.sleep(10000);
            System.out.println("After "+(i+1)*10+" seconds:");
         } catch(InterruptedException ie) {
            // Ignore
         }
         mlf.printStats();
      }

      // After this minute of activity, close the Connection and the
      // ServerSessionPool and quit
      conn.close();
      ssp.close();
   }

   /**************************************************************************
    *  parseArguments - Parse the given argument array, updating member      *
    *                   variables as we go. Terminates the application if    *
    *                   an error is encountered.                             *
    **************************************************************************/
   private static void parseArguments( String[] args ) {
      try {
         for( int i=0; i<args.length; i++ ) {
            String arg = args[i].toUpperCase();

            if( arg.equals("-NOJNDI"))         useJNDI   = false;
            else if( arg.equals("-URL"))       url       = args[++i];
            else if( arg.equals("-ICF"))       icf       = args[++i];
            else if( arg.equals("-TCFLOOKUP")) tcfLookup = args[++i];
            else if( arg.equals("-TLOOKUP"))   tLookup   = args[++i];
            else if( arg.equals("-T"))         tName     = args[++i];
            else if( arg.equals("-QMGR"))      qMgrName  = args[++i];
            else if( arg.equals("-POOLSIZE"))  poolSize  = Integer.parseInt(
                                                                  args[++i]);
            else if( arg.equals("-BATCHSIZE")) batchSize = Integer.parseInt(
                                                                  args[++i]);
            else
               throw new Exception("Unrecognised argument ("+arg+")");
         }
      } catch( Exception e ) {
         System.out.println("Error parsing arguments: "+e+"\nQuitting.");
         System.exit(-1);
      }
   }

   /**************************************************************************
    *  obtainAdminObjects - Obtains the two administered objects we need and *
    *                       places references to them in the given array.    *
    *                       The method used (JNDI versus explicit construct- *
    *                       ion) is determined by the value of 'useJNDI'.    *
    *                                                                        *
    *                       Terminates the application if any problems are   *
    *                       encountered.                                     *
    **************************************************************************/
   private static void obtainAdminObjects( Object[] outArray ) {
      try {
         if( useJNDI ) {
            // Set up JNDI environment parameters
            Hashtable env = new Hashtable();
            env.put( Context.PROVIDER_URL, url );
            env.put( Context.INITIAL_CONTEXT_FACTORY, icf );

            // Access the JNDI namespace
            Context ctx = new InitialContext(env);

            // Obtain the objects
            outArray[0] = ctx.lookup(tcfLookup);
            outArray[1] = ctx.lookup(tLookup);

         } else {
            // Create the objects explicitly - we need to refer to the
            // MQJMS-specific classes now
            com.ibm.mq.jms.MQTopicConnectionFactory tcf =
                        new com.ibm.mq.jms.MQTopicConnectionFactory();
            tcf.setQueueManager(qMgrName);
            tcf.setBrokerQueueManager(qMgrName);

            com.ibm.mq.jms.MQTopic t = new com.ibm.mq.jms.MQTopic();
            t.setBaseTopicName(tName);

            outArray[0] = tcf;
            outArray[1] = t;
         }
      } catch( Exception e ) {
         System.out.println("Error obtaining admin objects: "+e+"\nQuitting.");
         System.exit(-1);
      }
   }
}

