/*****************************************************************************
 *                                                                           *
 *                Licensed Materials - Property of IBM Corp.                 *
 *                  5648-C60 (c) Copyright IBM Corp. 2000                    *
 *                                                                           *
 *       MQJMS Standalone Application Server Facilities (ASF) Samples        *
 *                                                                           *
 *                                                                           *
 *  FILENAME:     CountingMessageListenerFactory.java                        *
 *                                                                           *
 *  DESCRIPTION:  Defines both a 'CountingMessageListener' which implements  *
 *                javax.jms.MessageListener and its factory class, used to   *
 *                create instances of it.                                    *
 *                                                                           *
 *                Further information about these classes and their usage    *
 *                can be found in the 'Websphere MQ Using Java'              *
 *                documentation.                                             *
 *                                                                           *
 *  USAGE:        n/a                                                        *
 *                                                                           *
 *  HISTORY:      010131 - First released as part of SupportPac MA88 v2      *
 *                                                                           *
 *****************************************************************************/

import javax.jms.*;
import java.util.*;

public class CountingMessageListenerFactory implements MessageListenerFactory {

   private int number = 0;  // The number of listeners created by this factory
   private String name;                // An identifying name for this factory

   private Vector listeners = new Vector(); // References to all the listeners
                                                    // created by this factory

   /**************************************************************************
    *  setName/getName - Getter and setter methods for the 'name' of this    *
    *                    MessageListenerFactory                              *
    **************************************************************************/
   public void setName( String name ) { this.name = name; }
   public String getName()            { return name;      }

   /**************************************************************************
    *  createMessageListener - Creates and returns new instance of the       *
    *                          CountingMessageListener class (also defined   *
    *                          in this file: see below), giving it a name    *
    *                          made up of the name of this factory and the   *
    *                          number of listeners already created.          *
    **************************************************************************/
   public MessageListener createMessageListener() {
      String listenerName;
      int    i;

      // Create the listener name
      synchronized(this) {
         i = number;
         number++;
         listenerName = name+" "+number;
      }

      // Create the listener based on the name we created above, and add a
      // reference to it to this factory's vector of created listeners,
      // ensuring we have the required capacity
      CountingMessageListener cml = new CountingMessageListener(listenerName);
      synchronized(listeners) {
         listeners.setSize(Math.max(listeners.size(),i+1));
      }
      listeners.setElementAt(cml,i);

      // Finally return the listener
      return(cml);
   }

   /**************************************************************************
    *  printStats - Prints out the number of messages received by all the    *
    *               listeners created by this factory                        *
    **************************************************************************/
   public void printStats() {
      Enumeration e = listeners.elements();

      while(e.hasMoreElements()) {
         CountingMessageListener cml = (CountingMessageListener)e.nextElement();
         System.out.println( cml.getName()+": received "+cml.getCount());
      }
   }

   /**************************************************************************
    *  getTotal - Returns the total number of messages seen by all the       *
    *             listeners created by this factory                          *
    **************************************************************************/
   public int getTotal() {
      int ret = 0;
      Enumeration e = listeners.elements();

      while(e.hasMoreElements()) {
         CountingMessageListener cml = (CountingMessageListener)e.nextElement();
         ret+=cml.getCount();
      }
      return(ret);
   }
}

/*****************************************************************************
 *  CountingMessageListener - A simple implementation of the MessageListener *
 *                            interface which keeps track of the number of   *
 *                            messages it has been passed, but does nothing  *
 *                            with the message itself.                       *
 *****************************************************************************/
class CountingMessageListener implements MessageListener {
   String name;                          // This listener's identifying string
   int count=0;        // A count of the number of messages it has been passed

   /** Constructor - Savea this message listener's identity for future use */
   CountingMessageListener( String name ) { this.name = name; }

   /** onMessage - Implements the javax.jms.MessageListener interface. This
       implementation simply increments the count of messages processed and
       ignores the message. */
   public void onMessage( Message m ) { count++; }

   /** getName - Returns this message listener's identity */
   public String getName() { return name; }

   /** getCount - Returns the count of messages this listener has seen */
   public int getCount() { return count; }
}
